# This class is used to check recipes against public CVEs.
#
# In order to use this class just inherit the class in the
# local.conf file and it will add the cve_check task for
# every recipe. The task can be used per recipe, per image,
# or using the special cases "world" and "universe". The
# cve_check task will print a warning for every unpatched
# CVE found and generate a file in the recipe WORKDIR/cve
# directory. If an image is build it will generate a report
# in DEPLOY_DIR_IMAGE for all the packages used.
#
# Example:
#   bitbake -c cve_check openssl
#   bitbake core-image-sato
#   bitbake -k -c cve_check universe
#
# DISCLAIMER
#
# This class/tool is meant to be used as support and not
# the only method to check against CVEs. Running this tool
# doesn't guarantee your packages are free of CVEs.
#
# Variables below are named using the following convention:
# CVE_CHECK_ -> class prefix (always to use)
# _DIR -> complete directory path
# _FILE -> complete file path (including extension)
# _FILE_NAME -> file name with extension
# _FILE_NAME_BASE -> file name without extension (used when multiple extensions could be used)
# For example: CVE_CHECK_IMAGE_REPORT_FILE_NAME_BASE has "_FILE_NAME_BASE" so that's just the file name,
# without the extension, of the report file. And has "CVE_CHECK_" to indicate that this variable is
# from the cve-check class


# CHECK OPTIONS
# The product name that the CVE database uses defaults to BPN, but may need to
# be overridden per recipe (for example tiff.bb sets CVE_PRODUCT=libtiff).
CVE_PRODUCT ??= "${BPN}"
CVE_VERSION ??= "${PV}"

# set to "alphabetical" for version using single alphabetical character as increment release
CVE_VERSION_SUFFIX ??= ""

# Check for packages without CVEs (no issues or missing product name)
CVE_CHECK_COVERAGE ??= "1"

# Skip CVE Check for packages (PN)
CVE_CHECK_SKIP_RECIPE ?= ""

# Ingore the check for a given list of CVEs. If a CVE is found,
# then it is considered patched. The value is a string containing
# space separated CVE values:
#
# CVE_CHECK_IGNORE = 'CVE-2014-2524 CVE-2018-1234'
#
CVE_CHECK_IGNORE ?= ""


# DATABASE OPTIONS
CVE_CHECK_DB_DIR ?= "${DL_DIR}/CVE_CHECK"
CVE_CHECK_DB_FILE ?= "${CVE_CHECK_DB_DIR}/nvdcve_1.1.db"
CVE_CHECK_DB_FILE_LOCK ?= "${CVE_CHECK_DB_FILE}.lock"


# TEMPORARY FILES
CVE_CHECK_TXT_INDEX_FILE ?= "${TMPDIR}/cve-report-index_txt.txt"
CVE_CHECK_TXT_INDEX_DIR ?= "${TMPDIR}/cve-tmp-files_txt"

CVE_CHECK_JSON_INDEX_FILE ?= "${TMPDIR}/cve-report-index_json.txt"
CVE_CHECK_JSON_INDEX_DIR ?= "${TMPDIR}/cve-tmp-files_json"


# OUTPUT OPTIONS
# Output directory
CVE_CHECK_OUTPUT_DIR ?= "${LOG_DIR}/cve"

# File names without extension of the image and build reports
# Build reports should not contain image specific bb vars line IMAGE_NAME or IMAGE_LINK_NAME
CVE_CHECK_IMAGE_REPORT_FILE_NAME_BASE ?= "cve-report_${IMAGE_LINK_NAME}"
CVE_CHECK_BUILD_REPORT_FILE_NAME_BASE ?= "cve-report"

# Create a report for each recipe in the build
CVE_CHECK_CREATE_RECIPE_REPORTS ??= "1"

# Name of the cve check per recipe file. If this is changed be sure that every recipe has a different
# value otherwise they will override each other
CVE_CHECK_RECIPE_FILE_NAME_BASE ?= "${PN}"

# Create a report file for the whole build
CVE_CHECK_CREATE_BUILD_REPORT ??= "1"

# Create a report file for each image
CVE_CHECK_CREATE_IMAGE_REPORT ??= "1"

# If set patched CVEs will show in the reports
CVE_CHECK_REPORT_PATCHED ??= "1"

# If set bitbake will show a warning if unpatched CVEs are found
CVE_CHECK_SHOW_WARNINGS ??= "1"

# Warning: Disabling one of these options doesn't clear their output folders, disabling both won't produce any files.
# Provide text output
CVE_CHECK_FORMAT_TEXT ??= "1"
# Provide JSON output
CVE_CHECK_FORMAT_JSON ??= "1"


# LAYERS OPTIONS
# Layers to be excluded
CVE_CHECK_LAYER_EXCLUDELIST ??= ""

# Layers to be included
CVE_CHECK_LAYER_INCLUDELIST ??= ""


python do_cve_check () {
    """
    Check recipe for patched and unpatched CVEs
    """
    from oe.cve_check import get_patched_cves

    if os.path.exists(d.getVar("CVE_CHECK_DB_FILE")):
        try:
            patched_cves = get_patched_cves(d)
        except FileNotFoundError:
            bb.fatal("Failure in searching patches")
        ignored, patched, unpatched, status = check_cves(d, patched_cves)

        if unpatched and d.getVar("CVE_CHECK_SHOW_WARNINGS") == "1":
            bb.warn("Found unpatched CVE (%s)" % (" ".join(unpatched)))

        if patched or unpatched or (d.getVar("CVE_CHECK_COVERAGE") == "1" and status):
            cve_data = get_cve_info(d, patched + unpatched + ignored)
            cve_write_data(d, patched, unpatched, ignored, cve_data, status)
    else:
        bb.note("No CVE database found, skipping CVE check")

}

addtask cve_check before do_build
do_cve_check[depends] = "cve-update-db-native:do_fetch"
do_cve_check[nostamp] = "1"

python cve_check_write_image_report () {
    """
    After 'do_rootfs' task is executed, if CVE_CHECK_CREATE_IMAGE_REPORT is set
    a complete image report is created.
    This includes all the information contained in the recipe reports builded by the image.
    """
    if d.getVar('CVE_CHECK_CREATE_IMAGE_REPORT') == "1":
        from oe.rootfs import image_list_installed_packages_pn
        deploy_dir = d.getVar("DEPLOY_DIR_IMAGE")
        report_name_base = d.getVar("CVE_CHECK_IMAGE_REPORT_FILE_NAME_BASE")
        report_dir_name = "image_reports"
        recipes_filter = list(image_list_installed_packages_pn(d))

        if d.getVar("CVE_CHECK_FORMAT_TEXT") == "1":
            generate_report(d, report_dir_name, report_name_base, "txt", generate_text_report, link_override=deploy_dir, gen_filter=recipes_filter)

        if d.getVar("CVE_CHECK_FORMAT_JSON") == "1":
            generate_report(d, report_dir_name, report_name_base, "json",
                            generate_json_report, link_override=deploy_dir,
                            gen_filter=recipes_filter)
}

ROOTFS_POSTPROCESS_COMMAND:prepend = "${@'cve_check_write_image_report; ' if d.getVar('CVE_CHECK_CREATE_IMAGE_REPORT') == '1' else ''}"
do_rootfs[recrdeptask] += "${@'do_cve_check' if d.getVar('CVE_CHECK_CREATE_IMAGE_REPORT') == '1' else ''}"

python cve_create_build_report_handler () {
    """
    After the build is completed, if "CVE_CHECK_CREATE_BUILD_REPORT" is set
    a complete report is created including all CVEs recipe reports information in a single file.
    """
    if d.getVar("CVE_CHECK_CREATE_BUILD_REPORT") == "1":
        report_name_base = d.getVar("CVE_CHECK_BUILD_REPORT_FILE_NAME_BASE")
        build_reports_dir = "build_reports"

        if d.getVar("CVE_CHECK_FORMAT_TEXT") == "1":
            generate_report(d, build_reports_dir, report_name_base, "txt", generate_text_report)

        if d.getVar("CVE_CHECK_FORMAT_JSON") == "1":
            generate_report(d, build_reports_dir, report_name_base, "json", generate_json_report)
}

addhandler cve_create_build_report_handler
cve_create_build_report_handler[eventmask] = "bb.event.BuildCompleted"

python cve_check_cleanup () {
    """
    Delete temporary files on bitbake exit
    """
    bb.utils.remove(e.data.getVar("CVE_CHECK_TXT_INDEX_FILE"))
    bb.utils.remove(e.data.getVar("CVE_CHECK_TXT_INDEX_DIR"), recurse=True)
    bb.utils.remove(e.data.getVar("CVE_CHECK_JSON_INDEX_FILE"))
    bb.utils.remove(e.data.getVar("CVE_CHECK_JSON_INDEX_DIR"), recurse=True)
}

addhandler cve_check_cleanup
cve_check_cleanup[eventmask] = "bb.cooker.CookerExit"

def check_cves(d, patched_cves):
    """
    Connect to the NVD database and find unpatched cves.
    """
    from oe.cve_check import Version

    pn = d.getVar("PN")
    real_pv = d.getVar("PV")
    suffix = d.getVar("CVE_VERSION_SUFFIX")

    cves_unpatched = []
    cves_ignored = []
    cves_status = []
    cves_in_recipe = False
    # CVE_PRODUCT can contain more than one product (eg. curl/libcurl)
    products = d.getVar("CVE_PRODUCT").split()
    # If this has been unset then we're not scanning for CVEs here (for example, image recipes)
    if not products:
        return ([], [], [], [])
    pv = d.getVar("CVE_VERSION").split("+git")[0]

    # If the recipe has been skipped/ignored we return empty lists
    if pn in d.getVar("CVE_CHECK_SKIP_RECIPE").split():
        bb.note("Recipe has been skipped by cve-check")
        return ([], [], [], [])

    cve_ignore = d.getVar("CVE_CHECK_IGNORE").split()

    import sqlite3
    db_file = d.expand("file:${CVE_CHECK_DB_FILE}?mode=ro")
    conn = sqlite3.connect(db_file, uri=True)

    # For each of the known product names (e.g. curl has CPEs using curl and libcurl)...
    for product in products:
        cves_in_product = False
        if ":" in product:
            vendor, product = product.split(":", 1)
        else:
            vendor = "%"

        # Find all relevant CVE IDs.
        for cverow in conn.execute("SELECT DISTINCT ID FROM PRODUCTS WHERE PRODUCT IS ? AND VENDOR LIKE ?", (product, vendor)):
            cve = cverow[0]

            if cve in cve_ignore:
                bb.note("%s-%s ignores %s" % (product, pv, cve))
                cves_ignored.append(cve)
                continue
            elif cve in patched_cves:
                bb.note("%s has been patched" % (cve))
                continue
            # Write status once only for each product
            if not cves_in_product:
                cves_status.append([product, True])
                cves_in_product = True
                cves_in_recipe = True

            vulnerable = False
            ignored = False

            for row in conn.execute("SELECT * FROM PRODUCTS WHERE ID IS ? AND PRODUCT IS ? AND VENDOR LIKE ?", (cve, product, vendor)):
                (_, _, _, version_start, operator_start, version_end, operator_end) = row
                #bb.debug(2, "Evaluating row " + str(row))
                if cve in cve_ignore:
                    ignored = True

                if (operator_start == '=' and pv == version_start) or version_start == '-':
                    vulnerable = True
                else:
                    if operator_start:
                        try:
                            vulnerable_start =  (operator_start == '>=' and Version(pv,suffix) >= Version(version_start,suffix))
                            vulnerable_start |= (operator_start == '>' and Version(pv,suffix) > Version(version_start,suffix))
                        except:
                            bb.warn("%s: Failed to compare %s %s %s for %s" %
                                    (product, pv, operator_start, version_start, cve))
                            vulnerable_start = False
                    else:
                        vulnerable_start = False

                    if operator_end:
                        try:
                            vulnerable_end  = (operator_end == '<=' and Version(pv,suffix) <= Version(version_end,suffix) )
                            vulnerable_end |= (operator_end == '<' and Version(pv,suffix) < Version(version_end,suffix) )
                        except:
                            bb.warn("%s: Failed to compare %s %s %s for %s" %
                                    (product, pv, operator_end, version_end, cve))
                            vulnerable_end = False
                    else:
                        vulnerable_end = False

                    if operator_start and operator_end:
                        vulnerable = vulnerable_start and vulnerable_end
                    else:
                        vulnerable = vulnerable_start or vulnerable_end

                if vulnerable:
                    if ignored:
                        bb.note("%s is ignored in %s-%s" % (cve, pn, real_pv))
                        cves_ignored.append(cve)
                    else:
                        bb.note("%s-%s is vulnerable to %s" % (pn, real_pv, cve))
                        cves_unpatched.append(cve)
                    break

            if not vulnerable:
                bb.note("%s-%s is not vulnerable to %s" % (pn, real_pv, cve))
                patched_cves.add(cve)

        if not cves_in_product:
            bb.note("No CVE records found for product %s, pn %s" % (product, pn))
            cves_status.append([product, False])

    conn.close()

    if not cves_in_recipe:
        bb.note("No CVE records for products in recipe %s" % (pn))

    return (list(cves_ignored), list(patched_cves), cves_unpatched, cves_status)

def get_cve_info(d, cves):
    """
    Get CVE information from the database.
    """

    import sqlite3

    cve_data = {}
    db_file = d.expand("file:${CVE_CHECK_DB_FILE}?mode=ro")
    conn = sqlite3.connect(db_file, uri=True)

    for cve in cves:
        for row in conn.execute("SELECT * FROM NVD WHERE ID IS ?", (cve,)):
            cve_data[row[0]] = {}
            cve_data[row[0]]["summary"] = row[1]
            cve_data[row[0]]["scorev2"] = row[2]
            cve_data[row[0]]["scorev3"] = row[3]
            cve_data[row[0]]["modified"] = row[4]
            cve_data[row[0]]["vector"] = row[5]

    conn.close()
    return cve_data

def create_file_and_update_index(d, content, recipes_tmp_dir, index_file, extension):
    """
    Helper function used to create a file inside recipes_tmp_dir with content in it.
    Then update the index file with it's path.

    Args:
        d: Bitbake data store object.
        content: String with the temporary recipe report.
        recipes_tmp_dir: Path of the folder containing the temporary recipes reports.
        index_file: Path of the index file. The index file is used to save all the temporary recipes reports paths.
        extension: String of the file extension. (Like "txt" or "json")
    
    Returns:
        None. Side effects the temporary recipes report file creation and appends its path to the index.
    """
    import bb
    bb.utils.mkdirhier(recipes_tmp_dir)
    fragment_file_name = "%s.%s" % (d.getVar("PN"), extension)
    fragment_file = os.path.join(recipes_tmp_dir, fragment_file_name)

    with open(fragment_file, "w") as f:
        f.write(content)
    with open(index_file, "a+") as f:
        f.write("%s\n" % fragment_file)

def tmp_report_saver_json(d, content):
    """
    Helper function used to save temporary information used when
    assembling a complete image or build report.
    For JSON reports only.

    Args:
        d: Bitbake data store object.
        content: String that will be wrote to the output file.

    Returns:
        None. Side effect from create_file_and_update_index.
    """
    recipes_tmp_dir = d.getVar("CVE_CHECK_JSON_INDEX_DIR")
    index_file = d.getVar("CVE_CHECK_JSON_INDEX_FILE")
    create_file_and_update_index(d, content, recipes_tmp_dir, index_file, "json")

def tmp_report_saver_txt(d, content):
    """
    Helper function used to save temporary information used when
    assembling a complete image or build report.
    For txt reports only.

    Args:
        d: Bitbake data store object.
        content: String that will be wrote to the output file.

    Returns:
        None. Side effect from create_file_and_update_index.
    """
    recipes_tmp_dir = d.getVar("CVE_CHECK_TXT_INDEX_DIR")
    index_file = d.getVar("CVE_CHECK_TXT_INDEX_FILE")
    create_file_and_update_index(d, content, recipes_tmp_dir, index_file, "txt")

def save_cve_recipe_report(d, content, format, tmp_report_saver):
    """
    Save in a dedicated file the content if "CVE_CHECK_CREATE_RECIPE_REPORTS" is set.
    If a report flag is set (image or build level) then "tmp_report_saver" is executed passing "content",
    this should save all the information needed when composing the complete report later.

    Args:
        d: Bitbake data store object.
        content: String that will be wrote to the output file.
        format: String of the output format name. Used as file extension and as the name for the
            format specific output folder.
        tmp_report_saver: Function that takes (d, content) and saves `content` to a temporary file.
            This is used in case of image or build reports are enabled.

    Returns:
        None. Side effects the recipe report file creation and tmp_report_saver side effect (only in case 
        CVE_CHECK_CREATE_IMAGE_REPORT or CVE_CHECK_CREATE_BUILD_REPORT is set).
    """
    if d.getVar("CVE_CHECK_CREATE_RECIPE_REPORTS") == "1":

        recipe_file_name = "%s.%s" % (d.getVar("CVE_CHECK_RECIPE_FILE_NAME_BASE"), format)
        out_dir = d.getVar("CVE_CHECK_OUTPUT_DIR")
        recipe_reports_dir = os.path.join(out_dir, "recipes_reports/%s" % format)
        recipe_file = os.path.join(recipe_reports_dir, recipe_file_name)
        bb.utils.mkdirhier(os.path.dirname(recipe_file))
        with open(recipe_file, "w") as f:
            f.write(content)

    if d.getVar("CVE_CHECK_CREATE_IMAGE_REPORT") == "1" or d.getVar("CVE_CHECK_CREATE_BUILD_REPORT") == 1:
        tmp_report_saver(d, content)

def generate_txt_cve_recipe_report_content(d, patched, unpatched, ignored, cve_data):
    """
    Construct the recipe report content string from the cve raw data.

    Args:
        d: Bitbake data store object.
        patched: List of patched CVEs.
        unpatched: List of unpatched CVEs.
        ignored: List of ignored CVEs.
        cve_data: Dictionary containing all the CVEs data.

    Returns:
        Recipe report content string in txt format.
    """
    from oe.utils import get_current_recipe_layer

    layer = get_current_recipe_layer(d)
    nvd_link = "https://nvd.nist.gov/vuln/detail/"
    write_string = ""
    unpatched_cves = []
    report_all = d.getVar("CVE_CHECK_REPORT_PATCHED") == "1"

    for cve in sorted(cve_data):
        is_patched = cve in patched
        is_ignored = cve in ignored

        if (is_patched or is_ignored) and not report_all:
            continue
        
        write_string += "LAYER: %s\n" % layer
        write_string += "PACKAGE NAME: %s\n" % d.getVar("PN")
        write_string += "PACKAGE VERSION: %s%s\n" % (d.getVar("EXTENDPE"), d.getVar("PV"))
        write_string += "CVE: %s\n" % cve
        if is_ignored:
            write_string += "CVE STATUS: Ignored\n"
        elif is_patched:
            write_string += "CVE STATUS: Patched\n"
        else:
            unpatched_cves.append(cve)
            write_string += "CVE STATUS: Unpatched\n"
        write_string += "CVE SUMMARY: %s\n" % cve_data[cve]["summary"]
        write_string += "CVSS v2 BASE SCORE: %s\n" % cve_data[cve]["scorev2"]
        write_string += "CVSS v3 BASE SCORE: %s\n" % cve_data[cve]["scorev3"]
        write_string += "VECTOR: %s\n" % cve_data[cve]["vector"]
        write_string += "MORE INFORMATION: %s%s\n\n" % (nvd_link, cve)

    return write_string

def generate_json_cve_recipe_report_content(d, patched, unpatched, ignored, cve_data, cve_status):
    """
    Construct the recipe report content string from the cve raw data.

    Args:
        d: Bitbake data store object.
        patched: List of patched CVEs.
        unpatched: List of unpatched CVEs.
        ignored: List of ignored CVEs.
        cve_data: Dictionary containing all the CVEs data.
        cve_status: List of products with their CVE status.

    Returns:
        Recipe report content string in json format.
    """
    import json
    from oe.utils import get_current_recipe_layer

    layer = get_current_recipe_layer(d)
    output = {"version":"1", "package": []}
    nvd_link = "https://nvd.nist.gov/vuln/detail/"
    report_all = d.getVar("CVE_CHECK_REPORT_PATCHED") == "1"

    unpatched_cves = []

    product_data = []
    for s in cve_status:
        p = {"product": s[0], "cvesInRecord": "Yes"}
        if s[1] == False:
            p["cvesInRecord"] = "No"
        product_data.append(p)

    package_version = "%s%s" % (d.getVar("EXTENDPE"), d.getVar("PV"))
    package_data = {
        "name" : d.getVar("PN"),
        "layer" : layer,
        "version" : package_version,
        "products": product_data
    }
    cve_list = []

    for cve in sorted(cve_data):
        is_patched = cve in patched
        is_ignored = cve in ignored
        status = "Unpatched"
        if (is_patched or is_ignored) and not report_all:
            continue
        if is_ignored:
            status = "Ignored"
        elif is_patched:
            status = "Patched"
        else:
            # default value of status is Unpatched
            unpatched_cves.append(cve)

        issue_link = "%s%s" % (nvd_link, cve)

        cve_item = {
            "id" : cve,
            "summary" : cve_data[cve]["summary"],
            "scorev2" : cve_data[cve]["scorev2"],
            "scorev3" : cve_data[cve]["scorev3"],
            "vector" : cve_data[cve]["vector"],
            "status" : status,
            "link": issue_link
        }
        cve_list.append(cve_item)

    package_data["issue"] = cve_list
    output["package"].append(package_data)

    return json.dumps(output, indent=2)

def is_layer_checked(d):
    """
        Helper function used to check if the layer of the current recipe
        is not in the excluded list and it's in the included layers (only if set).

        Args:
            d: Bitbake data store object.

        Returns:
            True if the layer is ok, otherwise False.
    """
    from oe.utils import get_current_recipe_layer
    
    layer = get_current_recipe_layer(d)
    include_layers = d.getVar("CVE_CHECK_LAYER_INCLUDELIST").split()
    exclude_layers = d.getVar("CVE_CHECK_LAYER_EXCLUDELIST").split()

    if (include_layers and layer not in include_layers) or \
       (exclude_layers and layer in exclude_layers):
        return False
    else:
        return True

def cve_write_data(d, patched, unpatched, ignored, cve_data, status):
    """
    Checks if the layer of the current recipe is ok then calls the functions to generate and save
    the recipe reports in txt and JSON formats only if the relative flags are set.

    Args:
        d: Bitbake data store object.
        patched: List of patched CVEs.
        unpatched: List of unpatched CVEs.
        ignored: List of ignored CVEs.
        cve_data: Dictionary containing all the CVEs data.
        cve_status: List of products with their CVE status.

    Returns:
        None. Same side effect of save_cve_recipe_report.

    """
    if is_layer_checked(d):
        if d.getVar("CVE_CHECK_FORMAT_TEXT") == "1":
            txt_content = generate_txt_cve_recipe_report_content(d, patched, unpatched, ignored, cve_data)
            save_cve_recipe_report(d, txt_content, "txt", tmp_report_saver_txt)
        if d.getVar("CVE_CHECK_FORMAT_JSON") == "1":
            json_content = generate_json_cve_recipe_report_content(d, patched, unpatched, ignored, cve_data, status)
            save_cve_recipe_report(d, json_content, "json", tmp_report_saver_json)

def get_content_list(d, index_file, filter=None):
    """
    Given the index file path and a filter read all the files listed in the index.
    If filter is not None use it on the file name (without extension).

    Args:
        d: Bitbake data store object.
        index_file: Path of the index file.
        filter: List of product name, used to filter out the reports to include in the output list.

    Returns:
        List of strings representing the content of the files read.
    """
    output_list = []
    with open(index_file) as f:
        file_path = f.readline()
        while file_path:
            file_path = file_path.rstrip()
            # Get the file name without extension
            file_name = file_path.split("/")[-1].split(".")[0]
            if filter is None or (filter and file_name in filter):
                with open(file_path, "r") as j:
                    output_list.append(j.read())
            file_path = f.readline()

    return output_list

def generate_json_report(d, report_file, report_link, filter=None):
    """
    Generate the JSON reports (image or build level).
    Store the results in report_file and creates the link from report_link to that.
    The report can be filtered using a list of file names.

    Args:
        d: Bitbake data store object.
        report_file: Path of the report file to create.
        report_link: Path where to create the link to report_file.
        filter: List of product name, used to filter out the products to include in report.

    Returns:
        None. Side effects the report and link creation in json format.
    """
    if os.path.exists(d.getVar("CVE_CHECK_JSON_INDEX_FILE")):
        import json
        from oe.cve_check import cve_check_merge_jsons, update_symlinks

        index_file = d.getVar("CVE_CHECK_JSON_INDEX_FILE")
        report_dict = {"version":"1", "package": []}
        temp_content_list = get_content_list(d, index_file, filter)
        [cve_check_merge_jsons(report_dict, json.loads(s)) for s in temp_content_list]

        with open(report_file, "w") as f:
            json.dump(report_dict, f, indent=2)

        update_symlinks(report_file, report_link)

def generate_text_report(d, report_file, report_link, filter=None):
    """
    Generate the txt reports (image or build level).
    Store the results in report_file and creates the link from report_link to that.

    Args:
        d: Bitbake data store object.
        report_file: Path of the report file to create.
        report_link: Path where to create the link to report_file.
        filter: List of product name, used to filter out the products to include in report.

    Returns:
        None. Side effects the report and link creation in txt format.
    """

    if os.path.exists(d.getVar("CVE_CHECK_TXT_INDEX_FILE")):
        from oe.cve_check import update_symlinks
        index_file = d.getVar("CVE_CHECK_TXT_INDEX_FILE")
        report_out = "".join(get_content_list(d, index_file, filter))

        with open(report_file, "w") as f:
            f.write(report_out)

        update_symlinks(report_file, report_link)

def generate_report(d, report_dir, report_name_base, extension, generator_func, link_override=None, gen_filter=None):
    """
    Form the necessary paths and directory structures to call the generator_func, that generates and saves the report.
    Args:
        report_dir: Path of the subfolder that is created inside the out_dir. This will store all format folders (txt and json folders).
        report_name_base: Name of the report without the extension
        extension: String of the extension (txt or json are currently used)
        generator_func: Function used to generate the report. It takes the report output path, a link path to save the output and a list to use as a filter.
        link_override: Optional argument used to override the standard path of the link (inside CVE_CHECK_OUTPUT_DIR), this will be used
            instead of out_dir when forming the link path.
        gen_filter: Optional argument to pass as "filter" to generator_func. List of file names used by generator functions to filter
            the recipe included in the report

    Return:
        None. Same side effect as generator_func. Prints where the report link is located.
    """
    import bb
    import shutil
    import datetime

    out_dir = d.getVar("CVE_CHECK_OUTPUT_DIR")
    timestamp = datetime.datetime.now().strftime('%Y%m%d%H%M%S')
    build_reports_dir = os.path.join(out_dir, report_dir)

    report_file_name = "%s-%s.%s" % (report_name_base, timestamp, extension)
    report_folder_dir = os.path.join(build_reports_dir, extension)
    bb.utils.mkdirhier(report_folder_dir)
    report_file = os.path.join(report_folder_dir, report_file_name)
    
    if link_override is None:
        report_link = os.path.join(out_dir, "%s.%s" % (report_name_base, extension))
    else:
        report_link = os.path.join(link_override, "%s.%s" % (report_name_base, extension))

    generator_func(d, report_file, report_link, filter=gen_filter)

    bb.plain("Report created at: %s" % report_link)
